#!/bin/sh
#
# check_child_procs - Nagios plugin, measures the number of child processes
# args: process name
# by Michael Tosch

set -f
PATH=/usr/xpg4/bin:/bin:/usr/bin:/usr/sbin:/sbin
export PATH

print_usage() {
  echo "Usage: $0 [ -w min:max ] [ -c min:max ] [-u owner] [-U owner] [-i UID] [procname]"
}

# default thresholds
warn="0:10"
crit="0:40"
procname="crond?"
uopt=""
uid=""

# get args
while getopts "w:c:u:i:h" OPT
do
  case $OPT in
  w) warn=$OPTARG;;
  c) crit=$OPTARG;;
  u) uopt="-$OPT $OPTARG";;
  i) uid=$OPTARG;;
  h)
    echo "This plugin counts the child processes of a given process name"
    print_usage
    echo "
procname can be a regular expression (ERE),
and must exactly match the (short-)name, as shown by ps -e
If omitted it defaults to '$procname'
Options:
 -w min:max
    Exit with WARNING status if #children is not within min and max
    If omitted it defaults to $warn
 -c min:max
    Exit with CRITICAL status if #children is not within min and max
    If omitted it defaults to $crit
 -u
    effective owner (uid or user) of procname and children
 -i
    effective uid of procname, children can differ"
    exit 3
  ;;
  *) print_usage; exit 3;;
  esac
done

if [ $OPTIND -lt $# ]; then
  print_usage; exit 3
elif [ $OPTIND -eq $# ]; then
  eval procname=\${$#}
fi

comm=comm
if [ "`uname`" = "SunOS" ]
then
  # ps zone|scope option
  zone=`{ /sbin/zonename; } 2>/dev/null`
  zopt=${zone:+"-o zone="}
  # find short script names
  case $procname in
  ?????????*) ;;
  *) comm=fname;; 
  esac
fi

UNIX95=1 ps ${uopt:-"-e"} -o pid= -o ppid= -o $comm= -o uid= $zopt |
awk -v w="$warn" -v c="$crit" -v proc="$procname" -v uid="$uid" -v zone="$zone" '
BEGIN { procre="(^|/)"proc"$" }
($5!=zone) {next}
($3~procre && (uid==$4 || uid=="")) {
 if (!($2 in P)) { pid=$1; pname=$3; sub(".*/","",pname) }
 P[$1]
}
{ PP[$2]++ }
END {
 if(pid==0) { gsub("[|]",",",proc); print "CRITICAL:",proc,"not running"; exit 2 }
 pp=PP[pid]+0
 msg=sprintf ("%s[%d] has %d child processes",pname,pid,pp)
 split (w,W,":"); split (c,C,":")
 graph=sprintf ("|procs=%d;%s;%s;0;",pp,w,c)
 if (pp<C[1] || pp>C[2]) { print "CRITICAL:",msg,", not in range",C[1]":"C[2],graph; exit 2 }
 if (pp<W[1] || pp>W[2]) { print "WARNING:",msg,", not in range",W[1]":"W[2],graph; exit 1 }
 print "OK:",msg,graph; exit 0
}
'
